# Paged Archives plugin for Movable Type
# Allows addition of paging for statically built archives.
# supports Categories, date-based and index page
# Author: Alden Bates http://www.tetrap.com/

package MT::Plugin::PagedArchives;

use strict;
use MT;
use MT::Plugin;
use MT::Template::Context;
use Fcntl qw( :DEFAULT :flock );
use MT::Util qw( archive_file_for start_end_month );

use vars qw( $VERSION );
$VERSION = '1.4';

my $plugin;
eval {
    require MT::Plugin;
    $plugin = MT::Plugin->new({
        name => 'Paged Archives',
        description => '<MT_TRANS phrase="Provides a paging mechanism for archives">',
        doc_link => 'http://www.aldenbates.com/plugins/pagedarchives.html',
        author_name => 'Alden Bates',
        author_link => 'http://www.aldenbates.com/',
        version => $VERSION,
        config_template => \&template,
        settings => new MT::PluginSettings([
            ['pa_perpage', { Default => 0 }],
            ['pa_useindex', { Default => 0 }],
            ['pa_listbycount', { Default => 0 }]
        ]),
    });
    MT->add_plugin($plugin);

    $plugin->{help_url} = sub { 'http://www.aldenbates.com/plugins/%t.html' };
    MT::Template::Context->add_tag(PAEntry => sub {'<!-- ==PA-Entry== -->';});
    MT::Template::Context->add_tag(PAEnd => sub {
      my $ctx = shift;
      my $blog = $ctx->stash('blog');
      my $entrycount = get_entry_count($ctx);
      "<!-- ==PA-End== $VERSION:$entrycount -->";
    });
    MT::Template::Context->add_tag(PADate => sub {'<!-- ==PADateStart== -->';});
    MT::Template::Context->add_tag(PADateEnd => sub {'<!-- ==PADateEnd== -->';});
    MT::Template::Context->add_tag(PASettings => sub {
      my ($ctx, $args) = @_;
      my $blog = $ctx->stash('blog');
      my $entrycount = $args->{numofentries};
      my $perpage = $args->{entriesperpage};
      my $pagelistlimit = $args->{pagelistlimit};
      $pagelistlimit = 0 unless($pagelistlimit);
      my $builder = $ctx->{__stash}{builder};
      my $tokens = $builder->compile($ctx, $entrycount);
      $entrycount = $builder->build($ctx,$tokens);
      $ctx->stash('pa_epp',$perpage);
      $ctx->stash('pa_entrycount',$entrycount) if($entrycount);
      $ctx->stash('pa_pll',$pagelistlimit);
      '';
    });
    MT::Template::Context->add_tag(PAPageNum => sub {
      my $ctx = shift;
      my $pagecount = get_page_count($ctx);
      return 1 if($pagecount==1);
      '<!-- ==PAPageNum== -->';
    });
    MT::Template::Context->add_tag(PANoIndexExtraPages => sub { '<!-- ==PANoIndex== -->'});
    MT::Template::Context->add_tag(PAPageList => sub { '<!-- ==PAPageList -->'});
    MT::Template::Context->add_tag(PAPreviousPage => sub {
      my ($ctx, $args) = @_;
      my $show = $args->{show};
      my $text = $args->{text};
      $show = '0' unless($show);
      $text = '&lt;&lt;' unless($text);
      "<!-- ==PAPreviousPage== $show \"$text\" -->";
    });
    MT::Template::Context->add_tag(PANextPage => sub {
      my ($ctx, $args) = @_;
      my $show = $args->{show};
      my $text = $args->{text};
      $show = '0' unless($show);
      $text = '&gt;&gt;' unless($text);
      "<!-- ==PANextPage== $show \"$text\" -->";
    });
    MT::Template::Context->add_tag(PAFirstPage => sub {
      my ($ctx, $args) = @_;
      my $show = $args->{show};
      my $text = $args->{text};
      $show = "0" unless($show);
      $text = "&lt;&lt;&lt;" unless($text);
      "<!-- ==PAFirstPage== $show \"$text\" -->";
    });
    MT::Template::Context->add_tag(PALastPage => sub {
      my ($ctx, $args) = @_;
      my $show = $args->{show};
      my $text = $args->{text};
      $show = '0' unless($show);
      $text = '&gt;&gt;&gt;' unless($text);
      "<!-- ==PALastPage== $show \"$text\" -->";
    });
    MT::Template::Context->add_tag(PAPageCount => sub {
      my $ctx = shift;
      get_page_count($ctx);
    });
    MT::Template::Context->add_conditional_tag(PAIfMultiplePages => sub {
      my $ctx = shift;
      (get_page_count($ctx)>1);
    });
    MT::Template::Context->add_container_tag(PAEntryExtra => sub {
      my($ctx, $args) = @_;
      my $entrynum = $args->{entrynumber};
      my $builder = $ctx->stash('builder');
      my $tokens = $ctx->stash('tokens');
      defined(my $out = "<!-- PAEntryExtra $entrynum -->".$builder->build($ctx,$tokens).'<!-- PAEntryExtraEnd -->')
            or return '';
      return $out;
    });
    MT->add_callback('BuildFile', 11, $plugin, \&endpa_hdlr);

    MT::Template::Context->add_tag(PAEntryLink => \&pa_entry_link_hdlr);
};

sub template {
    my $tmpl = <<'EOT';
    <div class="setting">
    <div class="label">
    <label for="pa_perpage"><MT_TRANS phrase="Entries per page:"></label>
    </div>
    <div class="field">
    <ul><li><input type="edit" name="pa_perpage" id="pa_perpage" value="<TMPL_VAR NAME=PA_PERPAGE>" /></li>
    </ul>
    </div>
    </div>
    <div class="setting">
    <div class="label">
    <label for="pa_useindex"><MT_TRANS phrase="Use &quot;index&quot; in filenames:"></label>
    </div>
    <div class="field">
    <ul><li><input type="checkbox" name="pa_useindex" id="pa_useindex" <TMPL_IF NAME=PA_USEINDEX>checked="checked"</TMPL_IF> /> <MT_TRANS phrase="Additional pages are named index2.html instead of 2.html. The index basename can be set in the Publishing settings."></li>
    </ul>
    </div>
    </div>
    <div class="setting">
    <div class="label">
    <label for="pa_listbycount"><MT_TRANS phrase="Use post numbers instead of page numbers:"></label>
    </div>
    <div class="field">
    <ul><li><input type="checkbox" name="pa_listbycount" id="pa_listbycount" <TMPL_IF NAME=PA_LISTBYCOUNT>checked="checked"</TMPL_IF> /> <MT_TRANS phrase="Uses alternate format for MTPAPageList. For instance: 1-10 11-20 instead of 1 2."></li>
    </ul>
    </div>
    </div>
EOT
}

# Works out how many entries are on the page we're building.
sub get_entry_count {
  my $ctx = shift;
  my $archivetype = $ctx->{ArchiveType};
  my $blog = $ctx->stash('blog');
  my $entrycount = $ctx->stash('pa_entrycount');
  $entrycount = MT::Template::Tags::Archive::_hdlr_archive_count($ctx) if(!$entrycount);
  if(!$entrycount) {
    $entrycount=$blog->entries_on_index;
    my $blogentrycount = MT::Template::Tags::Entry::_hdlr_blog_entry_count($ctx);
    $entrycount=$blogentrycount if($entrycount>$blogentrycount);
  }
  $entrycount;
}

sub get_page_count {
  my $ctx = shift;
  my $blog = $ctx->stash('blog');
  my $perpage = $ctx->stash('pa_epp');
  $perpage=$plugin->get_config_value('pa_perpage','blog:'.$blog->id) if(!$perpage);
  my $config = $plugin->get_config_hash();
  $perpage=$config->{'pa_perpage'} if(!$perpage);
  $perpage = 10 if(!$perpage);
  my $entrycount = get_entry_count($ctx);
  my $pagecount = $entrycount/$perpage+(($entrycount%$perpage)?1:0);
  int($pagecount);
}

# makes the "1 2 3 4" list of links to pages
# the current page is printed bold rather than linked
sub make_pagelist {
  my ($ctx, $pagenum, $baseurl, $perpage, $entrycount) = @_;
  my $pagecount = get_page_count($ctx);
  my $blog = $ctx->stash('blog');
  my $pagebasename = $ctx->stash('pa_pagebasename');
  my $ext = $blog->file_extension;
  $ext = '.'.$ext if($ext);
  my $pagelist='';
  my $spacer='';
  my $filename=$baseurl;
  my $pagelistlimit = $ctx->stash('pa_pll');
  my $bypostnumber = $plugin->get_config_value('pa_listbycount','blog:'.$blog->id);
  my ($beginpage,$endpage);
  if($pagelistlimit) {
    if($pagecount<=$pagelistlimit) {
    $beginpage=1; $endpage=$pagecount;
    } else {
      $beginpage=$pagenum-int(($pagelistlimit-1)/2);
      if($beginpage<1) {
        $beginpage=1;
        $endpage=$pagelistlimit;
      } else {
        $endpage=$pagenum+int($pagelistlimit/2);
        if($endpage>$pagecount) {
          $endpage=$pagecount;
          $beginpage=$pagecount-$pagelistlimit+1;
        }
      }
    }
  } else {
    $beginpage=1; $endpage=$pagecount;
  }

  $pagelist.='... ' if($beginpage>1);
  my $pagetext;
  for(my $ii=$beginpage; $ii<=$endpage; $ii++) {
    $filename=$pagebasename.$ii.$ext if($ii>1);
    $pagetext=$ii;
    if($bypostnumber) {
      my $firstpost=(($ii+$beginpage-2)*$perpage+1);
      my $lastpost=(($ii+$beginpage-1)*$perpage);
      $lastpost=$entrycount if($lastpost>$entrycount);
      $pagetext=$firstpost.(($lastpost!=$firstpost)?'-'.$lastpost:'');
    }
    $pagelist.=$spacer.'<a href="'.$filename.'">'.$pagetext.'</a>' if($ii != $pagenum);
    $pagelist.=$spacer.'<span class="pacurrent" style="font-weight:bold">'.$pagetext.'</span>' if($ii == $pagenum);
    $spacer=' ';
  }
  $pagelist.=' ...' if($endpage<$pagecount);
  $pagelist;
}

# BuildFile callback, executes after file has been constructed and written
sub endpa_hdlr {
  my $eh = shift;
  my %args = @_;
  my $at = $args{'ArchiveType'};
  return -1 if($at eq 'Individual');
  my $ctx = $args{'Context'};
  my $finfo = $args{'FileInfo'};
  my $blog = $args{'Blog'};
  my $file = $args{'File'};
  my $html = $args{'RawContent'};

  # abort if user hasn't added required tags
  return 1 if(($$html !~ /<!-- ==PA-Entry== -->/i)||($$html !~ /<!-- ==PA-End==/i));

  my $perpage;
  $perpage = $ctx->stash('pa_epp');
  $perpage=$plugin->get_config_value('pa_perpage','blog:'.$blog->id) if(!$perpage);
  my $config = $plugin->get_config_hash();
  $perpage = $config->{'pa_perpage'} if(!$perpage);
  $perpage=10 if(!$perpage);

  require File::Basename;
  my ($cffilename,$path) = File::Basename::fileparse($file);
  my $basename = MT->instance->config('IndexBasename');
  my $pagebasename = '';
  my $ext = $blog->file_extension;
  $ext = '.'.$ext if($ext);
  $basename=$1 if($cffilename =~ /(.*)$ext/);
  my $baseurl;
  $baseurl = $finfo->url if($finfo);                                             # We can get the URL from $finfo if we were given one.
  if($baseurl eq '') {                                                           # otherwise construct one the same way WeblogPublisher.pm does
    my $url;
    if($at eq 'index') {
      my $tmpl = $args{'Template'};
      $url = $blog->site_url.(($blog->site_url =~ m|/$|)?'':'/').$tmpl->outfile;
    } else {
      $url = $blog->archive_url;
      $url .= '/' unless ($url =~ m|/$|);
      my $map = $args{'TemplateMap'};
      $url .= $map->{__saved_output_file};
    }
    $baseurl = $url;
  }
  $baseurl =~ s|/([^/]*?)$ext$|/|g;
  my $useindex = $plugin->get_config_value('pa_useindex','blog:'.$blog->id);
  $pagebasename=$basename if($useindex);
  $ctx->stash("pa_pagebasename",$pagebasename);

  # Extract the template top and bottom
  my $templatetop=$1 if($$html =~ /^(.*?)<!-- ==PA-Entry/is);
  my $templatebottom=$2 if($$html =~ /PA-End== (.*?) -->(.*)$/is);

  # Find the number of pages
  my $entrycount=get_entry_count($ctx);
  my $pagecount=int($entrycount/$perpage+(($entrycount%$perpage)?1:0));

  my $firstlink = $baseurl;
  my $lastlink = $pagebasename.$pagecount.$ext;

  # finally, build all the pages
  my $datehtml='';
  my $entry;
  for(my $ii=1; $ii<=$pagecount; $ii++) {
    my $filename=$basename.$ext;
    $filename=$pagebasename.$ii.$ext if($ii>1);
    my $filepath=$path.'/'.$filename;
    my $prevlink='';
    $prevlink=$baseurl if($ii==2);
    $prevlink=$pagebasename.($ii-1).$ext if($ii>2);
    my $nextlink='';
    $nextlink=$pagebasename.($ii+1).$ext if($ii<$pagecount);
    my $pagelist=make_pagelist($ctx,$ii,$baseurl,$perpage,$entrycount);
    my $posts='';
    for(my $jj=1; $jj<=$perpage; $jj++) {
      last if(!($$html =~ /Entry== -->(.*?)<!-- ==PA-/gis));
      $entry=$1;
      $datehtml=$1 if($entry =~ /==PADateStart== -->(.*?)<!-- ==PADateEnd==/s);
      $entry =~ s/<!-- ==PADateEnd== -->/$datehtml/g if(($jj==1)&&($entry !~ /<!-- ==PADateStart== -->/));
      $entry =~ s/<!-- ==PADateStart== -->//g; $entry =~ s/<!-- ==PADateEnd== -->//g;
      $entry =~ s/<!-- PAEntryExtra $jj -->(.*?)<!-- PAEntryExtraEnd -->/$1/gs;
      $entry =~ s/<!-- PAEntryExtra(.*?)<!-- PAEntryExtraEnd -->//gs;
      $posts.="$entry";
    }
    build_pa_page($ii,$filepath,$templatetop,$posts,$templatebottom,$prevlink,$nextlink,($ii>1)?$firstlink:'',($ii<$pagecount)?$lastlink:'',$pagelist);
  }

  # delete ($pagecount+1).$ext onwards, which may be necessary if entries per page has increased.
  my $ii=$pagecount+1;
  my $filepath=$path.'/'.$pagebasename.$ii.$ext;
  while(-e $filepath) {
    unlink($filepath);
    $ii++;
    $filepath=$path.'/'.$pagebasename.$ii.$ext;
  }
  $ii=2;
  if($pagebasename) { $pagebasename=''; } else { $pagebasename=$basename; }
  $filepath=$path.'/'.$pagebasename.$ii.$ext;
  while(-e $filepath) {
    unlink($filepath);
    $ii++;
    $filepath=$path.'/'.$pagebasename.$ii.$ext;
  }
  1;
}

sub build_pa_page {
  my ($pagenum,$pagepath,$templatetop,$posts,$templatebottom,$prevlink,$nextlink,$firstlink,$lastlink,$pagelist) = @_;
  my $cfg = MT::ConfigMgr->instance;
  my $charset = $cfg->PublishCharset;

  unlink($pagepath);
  sysopen(PAGE, $pagepath, O_RDWR|O_CREAT|O_TRUNC, oct $cfg->HTMLPerms);

  replace_pa_variables(\$templatetop,$pagenum,$prevlink,$nextlink,$firstlink,$lastlink,$pagelist);
  print PAGE Encode::encode($charset, $templatetop);

  print PAGE Encode::encode($charset, $posts);

  replace_pa_variables(\$templatebottom,$pagenum,$prevlink,$nextlink,$firstlink,$lastlink,$pagelist);
  print PAGE Encode::encode($charset, $templatebottom);

  close(PAGE);
}

sub replace_pa_variables {
  my ($template,$pagenum,$prevlink,$nextlink,$firstlink,$lastlink,$pagelist) = @_;
  $$template =~ s/<!-- ==PAPageNum== -->/$pagenum/g;
  $$template =~ s/<!-- ==PAPageList -->/$pagelist/g;
  if($firstlink eq "") {
    $$template =~ s/<!-- ==PAFirstPage== 0 "(.*?)" -->//g;
    $$template =~ s/<!-- ==PAFirstPage== 1 "(.*?)" -->/$1/g;
  } else {
    $$template =~ s/<!-- ==PAFirstPage== \d "(.*?)" -->/<a href="$firstlink" rel="prev">$1<\/a>/g;
  }
  if($prevlink eq "") {
    $$template =~ s/<!-- ==PAPreviousPage== 0 "(.*?)" -->//g;
    $$template =~ s/<!-- ==PAPreviousPage== 1 "(.*?)" -->/$1/g;
    $$template =~ s/<!-- ==PANoIndex== -->//g;
  } else {
    $$template =~ s/<!-- ==PAPreviousPage== \d "(.*?)" -->/<a href="$prevlink" rel="prev">$1<\/a>/g;
    $$template =~ s/<!-- ==PANoIndex== -->/<meta name="robots" content="noindex,noarchive" \/>/g;
  }
  if($nextlink eq "") {
    $$template =~ s/<!-- ==PANextPage== 0 "(.*?)" -->//g;
    $$template =~ s/<!-- ==PANextPage== 1 "(.*?)" -->/$1/g;
  } else {
    $$template =~ s/<!-- ==PANextPage== \d "(.*?)" -->/<a href="$nextlink" rel="next">$1<\/a>/g;
  }
  if($lastlink eq "") {
    $$template =~ s/<!-- ==PALastPage== 0 "(.*?)" -->//g;
    $$template =~ s/<!-- ==PALastPage== 1 "(.*?)" -->/$1/g;
  } else {
    $$template =~ s/<!-- ==PALastPage== \d "(.*?)" -->/<a href="$lastlink" rel="next">$1<\/a>/g;
  }
}


sub pa_entry_link_hdlr {
  my ($ctx, $args) = @_;
  my $e = $ctx->stash('entry') or return $ctx->_no_entry_error('MTPAEntryLink');
  my $at = $args->{archive_type};
  my $blog = $ctx->stash('blog');
  my $arch = $blog->archive_url;
  $arch .= '/' unless $arch =~ m!/$!;
  my $perpage = $ctx->stash("pa_epp");
  $perpage=$plugin->get_config_value('pa_perpage','blog:'.$blog->id) if(!$perpage);
  my $config = $plugin->get_config_hash();
  $perpage=$config->{'pa_perpage'} if(!$perpage);
  $perpage = 10 if(!$perpage);
  my $archive_filename = '';
  my @objects;
  if($at eq 'Monthly') {
    $archive_filename = $e->archive_file('Monthly');
    # fetch entry count
    my($start, $end) = start_end_month($e->created_on, $blog);
    @objects = MT::Entry->load({
        blog_id => $e->blog_id,
        created_on => ([ $e->created_on, $end ]),
        status => MT::Entry::RELEASE()
    }, {
        'sort' => 'created_on',
        'direction' => 'ascend',
        'range_incl' => { 'created_on' => 1 }
    });
  } elsif($at eq 'Category') {
    $archive_filename = $e->archive_file('Category');
    my $cat = $e->{__category};
    @objects = MT::Entry->load({ blog_id => $blog->id,
                              status => MT::Entry::RELEASE(),
                              created_on => ([ $e->created_on, undef ]), },
                            { 'join' => [ 'MT::Placement', 'entry_id',
                              { category_id => $cat->id } ],
                              'range_incl' => { 'created_on' => 1 } });
  }
  if($archive_filename ne '') {
    my $pagenum = (int($#objects/$perpage))+1;
    # construct page file path
    if($pagenum==1) {
      my $link = $arch . $archive_filename;
      $link = MT::Util::strip_index($link, $blog) unless $args->{with_index};
      $link;
    } else {
      my $link = $arch . $archive_filename;
      $link = MT::Util::strip_index($link, $blog);
      my $useindex = $plugin->get_config_value('pa_useindex','blog:'.$blog->id);
      $link .= MT->instance->config('IndexBasename') if($useindex);
      my $ext = $blog->file_extension;
      $link .= $pagenum.'.'.$ext;
      $link;
    }
  }
}

1;
